local ToyPlus = LibStub("AceAddon-3.0"):NewAddon("ToyPlus", "AceConsole-3.0")
local icon = LibStub("LibDBIcon-1.0")

local ToyPlusLDB = LibStub("LibDataBroker-1.1"):NewDataObject("ToyPlus", {
	type = "launcher",
	text = "ToyPlus",
	icon = "Interface\\Icons\\INV_Misc_Toy_09",
	OnClick = function(self, button)
		if button == "RightButton" then InterfaceOptionsFrame_OpenToCategory("ToyPlus");InterfaceOptionsFrame_OpenToCategory("ToyPlus")--Workaround
		else ToyPlus:Launch() end
	end,
	OnTooltipShow = function(tt)
		tt:AddLine("ToyPlus")
		tt:AddLine("|cffffffff" .. "Left-Click to toggle window. Right-Click to open options.")
	end,
})


function ToyPlus:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("ToyPlusDB", {
		profile = {
			minimap = {
				hide = false,
			},
		},
	})
	icon:Register("ToyPlus", ToyPlusLDB, self.db.profile.minimap)
	if not ToyPlus.db.profile.rows then ToyPlus.db.profile.rows = 2 end
	ToyPlus:RegisterChatCommand("toyplus", "Launch")
	ToyPlus:RegisterOptions()
end

function ToyPlus:RegisterOptions()-- Blizzard Options
	local AceConfig = LibStub("AceConfig-3.0")
	AceConfig:RegisterOptionsTable("ToyPlus", {
		type = 'group',
		args = {
			togglebutton = {
				type = 'toggle',
				order = 1,
				name = "Show minimap icon",
				desc = "Show/hide minimap icon.",
				get = function()
					return not ToyPlus.db.profile.minimap.hide
				end,
				set = ToyPlus.ToggleLDBIcon,
			},
			toyRows = {
				type = 'select',
				style = 'radio',
				order = 2,
				name = "Number of Icon Rows:",
				values = {1, 2},
				get = function(info)
					return ToyPlus.db.profile.rows
				end,
				set = function(info, key)
					ToyPlus.db.profile.rows = key
					ToyPlus:ToggleRows()
				end,
			},
		},
	})
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("ToyPlus")
end

function ToyPlus:ToggleLDBIcon()-- Toggle Minimap Icon
	ToyPlus.db.profile.minimap.hide = not ToyPlus.db.profile.minimap.hide
	if ToyPlus.db.profile.minimap.hide then
		icon:Hide("ToyPlus")
	else
		icon:Show("ToyPlus")
	end
end

function ToyPlus:ToggleRows()-- Toggle Icon Rows
	local fra = _G["ToyPlusFrame"]
	if ToyPlus.db.profile.rows == 1 then
		for i = 6, 10 do
			local toyIcon = _G["ToyPlus_Icon"..i]
			if toyIcon then toyIcon:Hide() end
		end
		if fra then fra:SetHeight(82) end
	else
		for i = 6, 10 do
			local toyIcon = _G["ToyPlus_Icon"..i]
			if toyIcon then toyIcon:Show() end
		end
		if fra then fra:SetHeight(120) end
	end

end

function ToyPlus:RefreshToys() -- Add Toys to DB
	ToyPlusDB.toyName, ToyPlusDB.itemID, ToyPlusDB.toyIcon = {}, {}, {}
	local toyTotal = C_ToyBox.GetNumTotalDisplayedToys()
	local toyTotalLearned = C_ToyBox.GetNumLearnedDisplayedToys()
	local toyNum = 1
	if toyTotalLearned > 0 then
		for i = 1, toyTotal do
			local itemNo = C_ToyBox.GetToyFromIndex(i)
			local itemID, toyName, toyIcon, toyFave = C_ToyBox.GetToyInfo(itemNo)
			if PlayerHasToy(itemID) then
				ToyPlusDB.toyName[toyNum] = toyName
				ToyPlusDB.itemID[toyNum] = itemID
				ToyPlusDB.toyIcon[toyNum] = toyIcon
				toyNum=toyNum+1
			end
		end
	end
end

function ToyPlus:Launch()
	if C_ToyBox.GetToyFromIndex(1) > 0 then
		ToyPlus:RefreshToys()
		local fra = _G["ToyPlusFrame"]
		local fra2 = _G["ToyPlus_Popout"]
		if not fra then
			local fra = CreateFrame("Frame", "ToyPlusFrame", UIParent) --Toy Icons Frame
			fra:SetFrameStrata("BACKGROUND")
			fra:SetMovable(true); fra:EnableMouse(true)
			fra:RegisterForDrag("LeftButton")
			fra:SetScript("OnDragStart", fra.StartMoving)
			fra:SetScript("OnDragStop", fra.StopMovingOrSizing)
			fra:SetPoint("CENTER") 
			fra:SetSize(196,120)
			fra.texture = fra:CreateTexture()
			fra.texture:SetPoint("TOPLEFT", fra, 2, -2)
			fra.texture:SetPoint("BOTTOMRIGHT", fra, -2, 2)
			fra.texture:SetTexture(0,0,0,0.8)
			fra:Show()
			local backdrop = {
			  edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
			  tile = false, tileSize = 8, edgeSize = 8,
			  insets = { left = 2, right = 2, top = 2, bottom = 2 }
			}
			fra:SetBackdrop(backdrop)
			fra:SetBackdropBorderColor(0,0.564,1,1)
			local ttl = fra:CreateFontString("$parent_HeaderText","OVERLAY","GameFontNormal")--Title
			ttl:SetPoint("TOPLEFT",fra,"TOPLEFT",7,-6)
			ttl:SetText("ToyPlus")
			ttl:Show()
			local label = fra:CreateFontString("$parent_Label","OVERLAY","GameFontNormal")--Title
			label:SetPoint("BOTTOMRIGHT",fra,"BOTTOMRIGHT",-28,8)
			label:SetText("Toy List")
			label:Show()
			local btn = CreateFrame("BUTTON", "$parent_Close", fra, "UIPanelCloseButton")-- Close Button
			btn:SetPoint("TOPRIGHT", fra, "TOPRIGHT", 3, 3)
			btn:SetSize(30,30)
			btn:Show(); btn:Enable()
			btn:SetScript("OnClick", function (self, button, down) 
				if button == "LeftButton" then 
					fra:Hide()
					local fra2 = _G["ToyPlus_Popout"]
					if fra2 then
						fra2:Hide(); _G["ToyPlusFrame_Popout"].texture:SetTexture("Interface\\BUTTONS\\UI-SpellbookIcon-NextPage-Up")
					end
				end 
			end)
			local btnAddRow = CreateFrame("BUTTON", "$parent_AddRow", fra)-- Add Row Button
			btnAddRow:SetPoint("BOTTOMLEFT", fra, "BOTTOMLEFT", 8, 6)
			btnAddRow:SetAlpha(1); btnAddRow:Show(); btnAddRow:Enable()
			btnAddRow:SetSize(16,16)
			btnAddRow.texture = btnAddRow:CreateTexture()
			btnAddRow.texture:SetAllPoints(btnAddRow)
			btnAddRow.texture:SetDrawLayer("ARTWORK", -1)
			if ToyPlus.db.profile.rows == 1 then btnAddRow.texture:SetTexture("Interface\\PaperDollInfoFrame\\Character-Plus")
			else btnAddRow.texture:SetTexture("Interface\\WorldMap\\Dash_64") end
			btnAddRow:SetScript("OnClick", function (self, button) 
				if button == "LeftButton" then
					if ToyPlus.db.profile.rows == 2 then
						ToyPlus.db.profile.rows = 1
						btnAddRow.texture:SetTexture("Interface\\PaperDollInfoFrame\\Character-Plus")
					else
						ToyPlus.db.profile.rows = 2
						btnAddRow.texture:SetTexture("Interface\\WorldMap\\Dash_64")
					end
					ToyPlus:ToggleRows()
				end
			end)
			local btnPopout = CreateFrame("BUTTON", "$parent_Popout", fra)-- Popout Button
			btnPopout:SetPoint("BOTTOMRIGHT", fra, "BOTTOMRIGHT", -1, 1)
			btnPopout:SetAlpha(1); btnPopout:Show(); btnPopout:Enable()
			btnPopout:SetSize(26,26)
			btnPopout.texture = btnPopout:CreateTexture()
			btnPopout.texture:SetTexCoord(0,1,0,1)
			btnPopout.texture:SetAllPoints(btnPopout)
			btnPopout.texture:SetDrawLayer("ARTWORK", -1)
			btnPopout.texture:SetTexture("Interface\\BUTTONS\\UI-SpellbookIcon-NextPage-Up")
			btnPopout:SetScript("OnClick", function (self, button) 
				if button == "LeftButton" then
					local fra = _G["ToyPlus_Popout"]
					if not fra then
						ToyPlus:ShowList() 
						btnPopout.texture:SetTexture("Interface\\BUTTONS\\UI-SpellbookIcon-PrevPage-Up")
					elseif fra:IsVisible() then 
						btnPopout.texture:SetTexture("Interface\\BUTTONS\\UI-SpellbookIcon-NextPage-Up")
						ToyPlus:ShowList() 
					else
						btnPopout.texture:SetTexture("Interface\\BUTTONS\\UI-SpellbookIcon-PrevPage-Up")
						ToyPlus:ShowList()
					end
				end
			end)
			-- Toy Buttons
			local iconH = 8
			local iconV = -25
			for i = 1, 10 do
				if ToyPlusDB.toyName[i] then
					local toyName = ToyPlusDB.toyName[i]
					local toyIconStr = ToyPlusDB.toyIcon[i]
					local itemID = ToyPlusDB.itemID[i]
					toyIcon = CreateFrame("BUTTON","ToyPlus_Icon"..i,fra, "SecureActionButtonTemplate,ActionButtonTemplate")--Icons
					toyIcon:RegisterUnitEvent("ACTIONBAR_UPDATE_COOLDOWN")
					toyIcon:SetAttribute("type", "item")
					toyIcon:SetAttribute("item", toyName)
					toyIcon:RegisterForDrag("LeftButton")
					toyIcon:RegisterForClicks("LeftButtonUp", "RightButtonUp")
					toyIcon:SetSize(30,30)
					toyIcon:SetPoint("TOPLEFT",fra,"TOPLEFT",iconH,iconV)
					toyIcon.texture = toyIcon:CreateTexture()
					toyIcon.texture:SetTexCoord(0.1,0.9,0.1,0.9)
					toyIcon.texture:SetAllPoints(toyIcon)
					toyIcon.texture:SetDrawLayer("BACKGROUND")
					toyIcon.texture:SetTexture(toyIconStr)
					local border = _G["ToyPlus_Icon"..i.."NormalTexture"]
					border:SetVertexColor(0,0.6,1,1)
					local toyCD = CreateFrame("Cooldown","$parent_CD",toyIcon, "CooldownFrameTemplate")
					toyCD:SetAllPoints()
					local start, duration = GetItemCooldown(itemID)
					if start ~= 0 then toyCD:SetCooldown(start, duration) end
					toyIcon:SetScript("OnEnter", function(self)
						GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
						GameTooltip:SetToyByItemID(itemID)
					end)
					toyIcon:SetScript("OnLeave", function(self)
						GameTooltip:Hide()
					end)
					toyIcon:SetScript("OnDragStart", function(self)
						C_ToyBox.PickupToyBoxItem(itemID)
					end)
					toyIcon:SetScript("OnEvent", function(self, event, ...)
						if event == "ACTIONBAR_UPDATE_COOLDOWN" then
							local start, duration = GetItemCooldown(itemID)
							if start ~= 0 then toyCD:SetCooldown(start, duration) end
						end
					end)
					if PlayerHasToy(itemID) then toyIcon.texture:SetDesaturated(false) else toyIcon.texture:SetDesaturated(true) end
					iconH=iconH+38
					if i == 5 then iconH=8; iconV=-62 end
				end
			end
			ToyPlus:ToggleRows()
		else
			if fra:IsVisible() then 
				local btnPopout = _G["ToyPlusFrame_Popout"]
				fra:Hide()
				btnPopout.texture:SetTexture("Interface\\BUTTONS\\UI-SpellbookIcon-NextPage-Up")
				if fra2 then fra2:Hide() end 
			else
				fra:Show() 
			end
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("ToyPlus: Error - Toy Box hasn't been loaded yet. You need to open the Toy Box (Shift+P) prior to running.", 0, 0.7, 1)
	end
end

function ToyPlus:ShowList()
	local fra = _G["ToyPlus_Popout"]
	local mainFra = _G["ToyPlusFrame"]
	if not fra then
		fra = CreateFrame("Frame", "ToyPlus_Popout", UIParent)
		fra:SetFrameStrata("BACKGROUND")
		fra:EnableMouse(true)
		fra:RegisterForDrag("LeftButton")
		fra:SetPoint("BOTTOMLEFT", mainFra, "BOTTOMRIGHT",-1,0)
		fra:SetSize(286,186)
		fra.texture = fra:CreateTexture()--Background
		fra.texture:SetPoint("TOPLEFT", fra, 2, -2)
		fra.texture:SetPoint("BOTTOMRIGHT", fra, -2, 2)
		fra.texture:SetTexture(0,0,0)
		fra.texture:SetAlpha(0.8)
		fra:Show()
		local scrFra = CreateFrame("ScrollFrame", "ToyPlus_ScrollFrame", fra)
		scrFra:SetPoint("TOPLEFT", 2, -2)
		scrFra:SetPoint("BOTTOMRIGHT", -2, 2)
		fra.scrollframe = scrFra
		local scrBar = CreateFrame("Slider", "ToyPlus_Slider", scrFra, "UIPanelScrollBarTemplate")
		scrBar:SetPoint("TOPLEFT", fra, "TOPRIGHT", -19, -19)
		scrBar:SetPoint("BOTTOMLEFT", fra, "BOTTOMRIGHT", 19, 19)
		scrBar:SetMinMaxValues(1, 100)
		scrBar:SetValueStep(46)
		scrBar:SetValue(0)
		scrBar:SetWidth(16)
		scrBar:SetScript("OnValueChanged", function (self, value) self:GetParent():SetVerticalScroll(value) end)
		fra.scrollbar = scrBar 
		scrFra:EnableMouseWheel(true)
		scrFra:SetScript("OnMouseWheel",
			function(self, delta) 
				local x = scrBar:GetValue(); local y = scrBar:GetValueStep()
				if delta == 1 then scrBar:SetValue(x-y)
				else scrBar:SetValue(x+y) end 
		end)
		local scrContent = CreateFrame("Frame", "ToyPlus_Content", scrFra)
		scrContent:SetSize(128,128)
		scrFra.content = scrContent 
		scrFra:SetScrollChild(scrContent)
		local backdrop = { -- Border
		  edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		  tile = false, tileSize = 8, edgeSize = 8,
		  insets = { left = 2, right = 2, top = 2, bottom = 2 } }
		fra:SetBackdrop(backdrop)
		fra:SetBackdropBorderColor(0,0.564,1,1)
		--Checkboxes
		local x, y = 1, 1
		for i = 1, #ToyPlusDB.toyName do
			local itemNo = ToyPlusDB.itemID[i]
			local itemID, toyName, toyIconStr, toyFave = C_ToyBox.GetToyInfo(itemNo)
			local chkbox = CreateFrame("CheckButton", "ToyPlus_Check"..i, scrContent, "UICheckButtonTemplate")
			local chktxt =  _G[chkbox:GetName().."Text"]
			chkbox:SetSize(25, 25)
			chktxt:SetJustifyH("Left")
			if C_ToyBox.GetIsFavorite(itemID) then chkbox:SetChecked(true) else chkbox:SetChecked(false) end
			chktxt:SetText(toyName)
			chkbox:SetPoint("TOPLEFT",scrContent,"TOPLEFT",x, y)
			chkbox:SetScript("OnClick", function(self, button)
				if button == "LeftButton" then 
					if chkbox:GetChecked(true) then ToyPlus:RefreshList(itemID, true)
					else ToyPlus:RefreshList(itemID, false) end
				end
			end)
			chkbox:SetScript("OnEnter", function(self)
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
				GameTooltip:SetToyByItemID(itemID)
			end)
			chkbox:SetScript("OnLeave", function(self)
				GameTooltip:Hide()
			end)
			y=y-18
		end
		if #ToyPlusDB.toyName > 10 then
			x = (#ToyPlusDB.toyName*18)-178
			scrBar:SetMinMaxValues(1, x)
			scrBar:SetValueStep((x/10)-1)
		else scrBar:Hide() end
	else
		if fra:IsVisible() then fra:Hide() else fra:Show() end
	end
end

function ToyPlus:RefreshList(itemID, toyFave)
	C_ToyBox.SetIsFavorite(itemID, toyFave)
	ToyPlus:RefreshToys()
	local chkID = 1
	for i = 1, #ToyPlusDB.toyName do
		local itemNo = ToyPlusDB.itemID[i]
		local itemID, toyName, toyIconStr, toyFave = C_ToyBox.GetToyInfo(itemNo)
		local chkbox = _G["ToyPlus_Check"..i]
		local chktxt =  _G[chkbox:GetName().."Text"]
		if C_ToyBox.GetIsFavorite(itemID) then chkbox:SetChecked(true) else chkbox:SetChecked(false) end
		chktxt:SetText(toyName)
		chkbox:SetScript("OnClick", function(self, button)
			if button == "LeftButton" then 
				if chkbox:GetChecked(true) then ToyPlus:RefreshList(itemID, true)
				else ToyPlus:RefreshList(itemID, false) end
			end
		end)
		chkbox:SetScript("OnEnter", function(self)
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
			GameTooltip:SetToyByItemID(itemID)
		end)
		chkbox:SetScript("OnLeave", function(self)
			GameTooltip:Hide()
		end)
	end
	ToyPlus:RefreshIcons()
end

function ToyPlus:RefreshIcons()
	ToyPlus:RefreshToys()
	for i = 1, 10 do
		if ToyPlusDB.itemID[i] then
			local toyItemID = ToyPlusDB.itemID[i]
			local itemID, toyName, toyIconStr = C_ToyBox.GetToyInfo(toyItemID)
			local toyIcon = _G["ToyPlus_Icon"..i]
			local toyCD = _G["ToyPlus_Icon"..i.."_CD"]
			local start, duration = GetItemCooldown(itemID)
			toyCD:SetCooldown(start, duration)--Update CD
			toyIcon.texture:SetTexture(toyIconStr)
			toyIcon:SetAttribute("item", toyName)
			toyIcon:SetScript("OnEnter", function(self)
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
				GameTooltip:SetToyByItemID(itemID)
			end)
			toyIcon:SetScript("OnLeave", function(self)
				GameTooltip:Hide()
			end)
			toyIcon:SetScript("OnDragStart", function(self)
				C_ToyBox.PickupToyBoxItem(itemID)
			end)
			toyIcon:SetScript("OnEvent", function(self, event, ...)
				if event == "ACTIONBAR_UPDATE_COOLDOWN" then
					local start, duration = GetItemCooldown(itemID)
					if start ~= 0 then toyCD:SetCooldown(start, duration) end
				end
			end)
		end
	end
end